/*
 * flash_control.cc - libflashcontrol
 *
 * Copyright 2017 Sony Coporation
 */

#include <unistd.h>			// read(), close()
#include <sys/ioctl.h>		// ioctl, _IOR, _IOWR
#include <stdio.h>			// printf
#include <fcntl.h>			// O_RDWR
#include <string.h>			// memcpy
#include "flash_control.h"
#include "OSD_flash.h"

FlashControl::FlashControl()
{
	// Constructor
	m_File = -1;
}

FlashControl::~FlashControl()
{
	// Destructor
}

int FlashControl::getJedecId( uint32_t* const m_id, uint32_t* const d_id )
{
	struct flash_dev_ioctl_params param;
	int ret = 0;

	if( (m_id == nullptr) || (d_id == nullptr) ) {
		return FLASH_DRV_ERR;
	}

	param.cmd = FLASH_READ_JEDEC_ID;
	ret = ioctl( m_File, _IOWR( FLASH_IOCTL_MAGIC, 0, unsigned long ), &param );
	*m_id = (param.lvalue >> 16);
	*d_id = (param.lvalue & 0x0000FFFF);

	return ret;
}

int FlashControl::setCommand( const uint8_t cmd )
{
	struct flash_dev_ioctl_params param;
	int ret = 0;

	param.cmd = cmd;
	ret = ioctl( m_File, _IOWR( FLASH_IOCTL_MAGIC, 0, unsigned long ), &param );

	return ret;
}

int FlashControl::getData( const uint32_t addr )
{
	struct flash_dev_ioctl_params param;
	int ret = 0;

	param.cmd    = FLASH_NORMAL_READ_DATA;
	param.lvalue = addr;
	ret = ioctl( m_File, _IOWR( FLASH_IOCTL_MAGIC, 0, unsigned long ), &param );

	return ret;
}

int FlashControl::getStatus( const uint8_t cmd, uint32_t* const status )
{
	struct flash_dev_ioctl_params param;
	int ret = 0;

	if( status == nullptr ) {
		return FLASH_DRV_ERR;
	}

	param.cmd   = cmd;
	param.value = 0;
	ret = ioctl( m_File, _IOWR( FLASH_IOCTL_MAGIC, 0, unsigned long ), &param );
	*status = (uint32_t)param.value;

	return ret;
}

int FlashControl::setStatus( const uint8_t cmd, const uint32_t& status )
{
	struct flash_dev_ioctl_params param;
	int ret = 0;

	param.cmd   = cmd;
	param.value = status;
	ret = ioctl( m_File, _IOWR( FLASH_IOCTL_MAGIC, 0, unsigned long ), &param );

	return ret;
}

#define FLASH_DEV_PATH  "/dev/osd_flash_spi"

int FlashControl::spiOpen( void )
{
	int ret = 0;

	if( m_File <= 0 ) {
		m_File = open( FLASH_DEV_PATH, O_RDWR );
		if( m_File < 0 ) {
			ret = -1;
		}
	}

	return ret;
}

void FlashControl::spiClose( void )
{
	if( m_File > 0 ) {
		close( m_File );
		m_File = -1;
	}
}
